
# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '0.8.0'
version = '0.8.0'
full_version = '0.8.0'
git_revision = '75edeb6cc6ce095d6a5c4b1006c1c7756019bf0c'
release = True

if not release:
    version = full_version